@@ -13,6 +13,7 @@
 #include "soup.h"
 #include "soup-message-queue.h"
 #include "soup-socket-private.h"
+#include "soup-misc-private.h"
 
 typedef struct {
 	SoupSocket  *socket;
@@ -44,6 +45,7 @@ enum {
 	PROP_REMOTE_URI,
 	PROP_SOCKET_PROPERTIES,
 	PROP_STATE,
+	PROP_SSL,
 
 	LAST_PROP
 };
@@ -98,10 +100,6 @@ soup_connection_set_property (GObject *object, guint prop_id,
 	switch (prop_id) {
 	case PROP_REMOTE_URI:
 		priv->remote_uri = g_value_dup_boxed (value);
-		if (priv->remote_uri)
-			priv->ssl = (priv->remote_uri->scheme == SOUP_URI_SCHEME_HTTPS);
-		else
-			priv->ssl = FALSE;
 		break;
 	case PROP_SOCKET_PROPERTIES:
 		priv->socket_props = g_value_dup_boxed (value);
@@ -109,6 +107,9 @@ soup_connection_set_property (GObject *object, guint prop_id,
 	case PROP_STATE:
 		soup_connection_set_state (SOUP_CONNECTION (object), g_value_get_uint (value));
 		break;
+	case PROP_SSL:
+		priv->ssl = g_value_get_boolean (value);
+		break;
 	default:
 		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
 		break;
@@ -131,6 +132,9 @@ soup_connection_get_property (GObject *object, guint prop_id,
 	case PROP_STATE:
 		g_value_set_enum (value, priv->state);
 		break;
+	case PROP_SSL:
+		g_value_set_boolean (value, priv->ssl);
+		break;
 	default:
 		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
 		break;
@@ -190,6 +194,12 @@ soup_connection_class_init (SoupConnectionClass *connection_class)
 				   "Current state of connection",
 				   SOUP_TYPE_CONNECTION_STATE, SOUP_CONNECTION_NEW,
 				   G_PARAM_READWRITE));
+	g_object_class_install_property (
+		object_class, PROP_SSL,
+		g_param_spec_boolean (SOUP_CONNECTION_SSL,
+				      "Connection uses TLS",
+				      "Whether the connection should use TLS",
+				      FALSE, G_PARAM_READWRITE));
 }
 
 static void
@@ -621,6 +631,46 @@ soup_connection_is_via_proxy (SoupConnection *conn)
 	return priv->proxy_uri != NULL;
 }
 
+static gboolean
+is_idle_connection_disconnected (SoupConnection *conn)
+{
+	SoupConnectionPrivate *priv = soup_connection_get_instance_private (conn);
+	GIOStream *iostream;
+	GInputStream *istream;
+	char buffer[1];
+	GError *error = NULL;
+
+	if (!soup_socket_is_connected (priv->socket))
+		return TRUE;
+
+	if (priv->unused_timeout && priv->unused_timeout < time (NULL))
+		return TRUE;
+
+	iostream = soup_socket_get_iostream (priv->socket);
+	istream = g_io_stream_get_input_stream (iostream);
+
+	/* This is tricky. The goal is to check if the socket is readable. If
+	 * so, that means either the server has disconnected or it's broken (it
+	 * should not send any data while the connection is in idle state). But
+	 * we can't just check the readability of the SoupSocket because there
+	 * could be non-application layer TLS data that is readable, but which
+	 * we don't want to consider. So instead, just read and see if the read
+	 * succeeds. This is OK to do here because if the read does succeed, we
+	 * just disconnect and ignore the data anyway.
+	 */
+	g_pollable_input_stream_read_nonblocking (G_POLLABLE_INPUT_STREAM (istream),
+						  &buffer, sizeof (buffer),
+						  NULL, &error);
+	if (!g_error_matches (error, G_IO_ERROR, G_IO_ERROR_WOULD_BLOCK)) {
+		g_clear_error (&error);
+		return TRUE;
+	}
+
+	g_error_free (error);
+
+	return FALSE;
+}
+
 SoupConnectionState
 soup_connection_get_state (SoupConnection *conn)
 {
@@ -631,12 +681,7 @@ soup_connection_get_state (SoupConnection *conn)
 	priv = soup_connection_get_instance_private (conn);
 
 	if (priv->state == SOUP_CONNECTION_IDLE &&
-	    (!soup_socket_is_connected (priv->socket) ||
-	     soup_socket_is_readable (priv->socket)))
-		soup_connection_set_state (conn, SOUP_CONNECTION_REMOTE_DISCONNECTED);
-
-	if (priv->state == SOUP_CONNECTION_IDLE &&
-	    priv->unused_timeout && priv->unused_timeout < time (NULL))
+	    is_idle_connection_disconnected (conn))
 		soup_connection_set_state (conn, SOUP_CONNECTION_REMOTE_DISCONNECTED);
 
 	return priv->state;
