@@ -1149,17 +1149,6 @@ soup_socket_steal_gsocket (SoupSocket *sock)
 	return gsock;
 }
 
-gboolean
-soup_socket_is_readable (SoupSocket *sock)
-{
-	SoupSocketPrivate *priv;
-
-	g_return_val_if_fail (SOUP_IS_SOCKET (sock), FALSE);
-	priv = soup_socket_get_instance_private (sock);
-
-	return g_pollable_input_stream_is_readable (G_POLLABLE_INPUT_STREAM (priv->istream));
-}
-
 GIOStream *
 soup_socket_get_connection (SoupSocket *sock)
 {
